@@ -2,6 +2,7 @@ use crate::services::{ServeDir, ServeFile};
 use brotli::BrotliDecompress;
 use bytes::Bytes;
 use flate2::bufread::{DeflateDecoder, GzDecoder};
+use http::header::ALLOW;
 use http::{header, Method, Response};
 use http::{Request, StatusCode};
 use http_body::Body as HttpBody;
@@ -585,8 +586,11 @@ async fn last_modified() {
 
 #[tokio::test]
 async fn with_fallback_svc() {
-    async fn fallback<B>(_: Request<B>) -> io::Result<Response<Body>> {
-        Ok(Response::new(Body::from("from fallback")))
+    async fn fallback<B>(req: Request<B>) -> io::Result<Response<Body>> {
+        Ok(Response::new(Body::from(format!(
+            "from fallback {}",
+            req.uri().path()
+        ))))
     }
 
     let svc = ServeDir::new("..").fallback(tower::service_fn(fallback));
@@ -600,7 +604,7 @@ async fn with_fallback_svc() {
     assert_eq!(res.status(), StatusCode::OK);
 
     let body = body_into_text(res.into_body()).await;
-    assert_eq!(body, "from fallback");
+    assert_eq!(body, "from fallback /doesnt-exist");
 }
 
 #[tokio::test]
@@ -634,4 +638,53 @@ async fn method_not_allowed() {
     let res = svc.oneshot(req).await.unwrap();
 
     assert_eq!(res.status(), StatusCode::METHOD_NOT_ALLOWED);
+    assert_eq!(res.headers()[ALLOW], "GET,HEAD");
+}
+
+#[tokio::test]
+async fn calling_fallback_on_not_allowed() {
+    async fn fallback<B>(req: Request<B>) -> io::Result<Response<Body>> {
+        Ok(Response::new(Body::from(format!(
+            "from fallback {}",
+            req.uri().path()
+        ))))
+    }
+
+    let svc = ServeDir::new("..")
+        .call_fallback_on_method_not_allowed(true)
+        .fallback(tower::service_fn(fallback));
+
+    let req = Request::builder()
+        .method(Method::POST)
+        .uri("/doesnt-exist")
+        .body(Body::empty())
+        .unwrap();
+    let res = svc.oneshot(req).await.unwrap();
+
+    assert_eq!(res.status(), StatusCode::OK);
+
+    let body = body_into_text(res.into_body()).await;
+    assert_eq!(body, "from fallback /doesnt-exist");
+}
+
+#[tokio::test]
+async fn with_fallback_svc_and_not_append_index_html_on_directories() {
+    async fn fallback<B>(req: Request<B>) -> io::Result<Response<Body>> {
+        Ok(Response::new(Body::from(format!(
+            "from fallback {}",
+            req.uri().path()
+        ))))
+    }
+
+    let svc = ServeDir::new("..")
+        .append_index_html_on_directories(false)
+        .fallback(tower::service_fn(fallback));
+
+    let req = Request::builder().uri("/").body(Body::empty()).unwrap();
+    let res = svc.oneshot(req).await.unwrap();
+
+    assert_eq!(res.status(), StatusCode::OK);
+
+    let body = body_into_text(res.into_body()).await;
+    assert_eq!(body, "from fallback /");
 }
