@@ -68,10 +68,15 @@ impl DefaultOnResponse {
 
     /// Set the [`Level`] used for [tracing events].
     ///
+    /// Please note that while this will set the level for the tracing events
+    /// themselves, it might cause them to lack expected information, like
+    /// request method or path. You can address this using
+    /// [`DefaultMakeSpan::level`].
+    ///
     /// Defaults to [`Level::DEBUG`].
     ///
     /// [tracing events]: https://docs.rs/tracing/latest/tracing/#events
-    /// [`Level::DEBUG`]: https://docs.rs/tracing/latest/tracing/struct.Level.html#associatedconstant.DEBUG
+    /// [`DefaultMakeSpan::level`]: crate::trace::DefaultMakeSpan::level
     pub fn level(mut self, level: Level) -> Self {
         self.level = level;
         self
@@ -108,6 +113,39 @@ macro_rules! log_pattern_match {
     ) => {
         match ($this.level, $include_headers, $this.latency_unit, status($res)) {
             $(
+                (Level::$level, true, LatencyUnit::Seconds, None) => {
+                    tracing::event!(
+                        Level::$level,
+                        latency = format_args!("{} s", $latency.as_secs_f64()),
+                        response_headers = ?$res.headers(),
+                        "finished processing request"
+                    );
+                }
+                (Level::$level, false, LatencyUnit::Seconds, None) => {
+                    tracing::event!(
+                        Level::$level,
+                        latency = format_args!("{} s", $latency.as_secs_f64()),
+                        "finished processing request"
+                    );
+                }
+                (Level::$level, true, LatencyUnit::Seconds, Some(status)) => {
+                    tracing::event!(
+                        Level::$level,
+                        latency = format_args!("{} s", $latency.as_secs_f64()),
+                        status = status,
+                        response_headers = ?$res.headers(),
+                        "finished processing request"
+                    );
+                }
+                (Level::$level, false, LatencyUnit::Seconds, Some(status)) => {
+                    tracing::event!(
+                        Level::$level,
+                        latency = format_args!("{} s", $latency.as_secs_f64()),
+                        status = status,
+                        "finished processing request"
+                    );
+                }
+
                 (Level::$level, true, LatencyUnit::Millis, None) => {
                     tracing::event!(
                         Level::$level,
@@ -224,7 +262,7 @@ impl<B> OnResponse<B> for DefaultOnResponse {
 }
 
 fn status<B>(res: &Response<B>) -> Option<i32> {
-    use crate::classify::ParsedGrpcStatus;
+    use crate::classify::grpc_errors_as_failures::ParsedGrpcStatus;
 
     let is_grpc = res
         .headers()
@@ -232,7 +270,10 @@ fn status<B>(res: &Response<B>) -> Option<i32> {
         .map_or(false, |value| value == "application/grpc");
 
     if is_grpc {
-        match crate::classify::classify_grpc_metadata(res.headers()) {
+        match crate::classify::grpc_errors_as_failures::classify_grpc_metadata(
+            res.headers(),
+            crate::classify::GrpcCode::Ok.into_bitmask(),
+        ) {
             ParsedGrpcStatus::Success
             | ParsedGrpcStatus::HeaderNotString
             | ParsedGrpcStatus::HeaderNotInt => Some(0),
